@@ -538,9 +538,9 @@ class ValveStackChainTest(ValveTestBases.ValveTestSmall):
         self.activate_all_ports()
         self.validate_edge_learn_ports()
 
-    def test_stack_learn_root(self):
-        """Test stack learned root"""
-        self.update_config(self._config_edge_learn_stack_root(True))
+    def test_stack_learn_not_root(self):
+        """Test stack learned when not root"""
+        self.update_config(self._config_edge_learn_stack_root(False))
         self.activate_all_ports()
         self.validate_edge_learn_ports()
 
@@ -617,6 +617,8 @@ class ValveStackEdgeLearnTestCase(ValveStackLoopTest):
     def test_edge_learn_edge_port(self):
         """Check the behavior of the basic edge_learn_port algorithm"""
 
+        self.update_config(self._config_edge_learn_stack_root(False))
+
         self.activate_all_ports()
 
         self.validate_edge_learn_ports()
@@ -631,8 +633,6 @@ class ValveStackEdgeLearnTestCase(ValveStackLoopTest):
     def test_edge_learn_stack_root(self):
         """Check the behavior of learning always towards stack root"""
 
-        self.update_config(self._config_edge_learn_stack_root(True))
-
         self.activate_all_ports()
 
         self.validate_edge_learn_ports()
