@@ -926,9 +926,9 @@ class ValveTestBases:
                 valve = self.valve
             port = valve.dp.ports[port_no]
             port.dyn_stack_current_state = status
-            valve.flood_manager.update_stack_topo(True, valve.dp, port)
+            valve.switch_manager.update_stack_topo(True, valve.dp, port)
             for valve_vlan in valve.dp.vlans.values():
-                self.apply_ofmsgs(valve.flood_manager.add_vlan(valve_vlan))
+                self.apply_ofmsgs(valve.switch_manager.add_vlan(valve_vlan))
 
         def set_stack_port_up(self, port_no, valve=None):
             """Set stack port up recalculating topology as necessary."""
